@@ -1,3 +1,4 @@
+import locale
 import os
 import sys
 import stat
@@ -12,7 +13,7 @@ from collections import deque, namedtuple
 
 from werkzeug.posixemulation import rename
 
-from lektor._compat import iteritems, text_type
+from lektor._compat import PY2, iteritems, text_type
 from lektor.context import Context
 from lektor.build_programs import builtin_build_programs
 from lektor.reporter import reporter
@@ -127,9 +128,9 @@ class BuildState(object):
         This can be usedful in some scenarious when building with external
         tools.
         """
-        dir = os.path.join(self.builder.meta_path, 'tmp')
+        tmpdir = os.path.join(self.builder.meta_path, 'tmp')
         try:
-            os.makedirs(dir)
+            os.makedirs(tmpdir)
         except OSError:
             pass
         fn = os.path.join(dir, 'nt-%s-%s.tmp' % (identifier or 'generic',
@@ -140,6 +141,7 @@ class BuildState(object):
     def get_file_info(self, filename):
         if filename:
             return self.path_cache.get_file_info(filename)
+        return None
 
     def to_source_filename(self, filename):
         return self.path_cache.to_source_filename(filename)
@@ -548,7 +550,7 @@ class VirtualSourceInfo(object):
 
         if self.path != other.path:
             raise ValueError("trying to compare mismatched virtual paths: "
-                             "%r.unchanged(%r)", self, other)
+                             "%r.unchanged(%r)" % (self, other))
 
         return (self.mtime, self.checksum) == (other.mtime, other.checksum)
 
@@ -603,9 +605,9 @@ class Artifact(object):
 
     def ensure_dir(self):
         """Creates the directory if it does not exist yet."""
-        dir = os.path.dirname(self.dst_filename)
+        dst_dir = os.path.dirname(self.dst_filename)
         try:
-            os.makedirs(dir)
+            os.makedirs(dst_dir)
         except OSError:
             pass
 
@@ -795,7 +797,7 @@ class Artifact(object):
         ctx = self.begin_update()
         try:
             yield ctx
-        except:
+        except:  # pylint: disable=bare-except
             exc_info = sys.exc_info()
             self.finish_update(ctx, exc_info)
         else:
@@ -933,7 +935,7 @@ class PathCache(object):
         return rv
 
 
-def process_build_flags(flags):
+def process_extra_flags(flags):
     if isinstance(flags, dict):
         return flags
     rv = {}
@@ -949,8 +951,8 @@ def process_build_flags(flags):
 class Builder(object):
 
     def __init__(self, pad, destination_path, buildstate_path=None,
-                 build_flags=None):
-        self.build_flags = process_build_flags(build_flags)
+                 extra_flags=None):
+        self.extra_flags = process_extra_flags(extra_flags)
         self.pad = pad
         self.destination_path = os.path.abspath(os.path.join(
             pad.db.env.root_path, destination_path))
@@ -983,7 +985,22 @@ class Builder(object):
 
     def connect_to_database(self):
         con = sqlite3.connect(self.buildstate_database_filename,
+                              isolation_level=None,
                               timeout=10, check_same_thread=False)
+        if PY2:
+            # This code block solve lektor/lektor#243 issue
+            # `os.walk` return :class:`str` type string. But :class:`str` mean
+            # different type between Python 2 and 3.
+            # (:class:`str` on PY2 is equivalent to :class:`bytes` on PY3)
+            # :mod:`sqlite` can not consume multibyte input string because
+            # it expect :class:`unicode` on PY2, not bytes.
+            # If sqlite make connection without text_factory, Multibyte
+            # filename must raise :class:`ProgrammingError`
+            # So we must decode filename as system encoding for on PY2 via
+            # text_factory.
+
+            system_encoding = locale.getdefaultlocale()[1]
+            con.text_factory = lambda x: x.decode(system_encoding, 'ignore')
         cur = con.cursor()
         cur.execute('pragma journal_mode=WAL')
         cur.execute('pragma synchronous=NORMAL')
@@ -1018,7 +1035,8 @@ class Builder(object):
                                   reversed(builtin_build_programs)):
             if isinstance(source, cls):
                 return builder(source, build_state)
-        raise RuntimeError('I do not know how to build %r' % source)
+        # TODO: re-enable pylint when https://github.com/PyCQA/pylint/issues/1782 is fixed.
+        raise RuntimeError('I do not know how to build %r' % source) # pylint: disable=inconsistent-return-statements
 
     def build_artifact(self, artifact, build_func):
         """Various parts of the system once they have an artifact and a
@@ -1041,6 +1059,7 @@ class Builder(object):
                         ctx.record_dependency(project_file)
                     build_func(artifact)
                 return ctx
+        return None
 
     def update_source_info(self, prog, build_state):
         """Updates a single source info based on a program.  This is done
